@@ -47,10 +47,10 @@
 
 #include "properties-window.h"
 
+#include "windows/format_dialog.h"
 #include "file-launch-manager.h"
 #include "file-launch-action.h"
 #include "file-lauch-dialog.h"
-
 #include "file-operation-error-dialog.h"
 #include "file-enumerator.h"
 #include "gerror-wrapper.h"
@@ -102,6 +102,10 @@ void DirectoryViewMenu::fillActions()
         m_is_computer = true;
     }
 
+    if (m_directory == "network:///") {
+        m_is_network = true;
+    }
+
     if (m_directory == "trash:///") {
         m_is_trash = true;
     }
@@ -122,10 +126,19 @@ void DirectoryViewMenu::fillActions()
         m_is_favorite = true;
     }
 
-    if (m_directory.startsWith("kydroid:///")) {
+    if (m_directory.startsWith("kydroid:///") || m_directory.startsWith("kmre:///")) {
         m_is_kydroid = true;
     }
 
+    if (m_directory.startsWith("ftp://")
+            || m_directory.startsWith("sftp://")) {
+        m_is_ftp = true;
+    }
+
+    if(m_directory == "filesafe:///") {
+        m_is_filesafe = true;
+    }
+
     QString desktop = QStandardPaths::writableLocation(QStandardPaths::DesktopLocation);
 
     for (auto uriIndex = 0; uriIndex < m_selections.count(); ++uriIndex) {
@@ -143,6 +156,10 @@ void DirectoryViewMenu::fillActions()
     if (!openActions.isEmpty())
         addSeparator();
 
+    //netwotk items not show operation menu
+    if (m_is_network)
+        return;
+
     if (! m_is_kydroid){
         //create template actions
         auto templateActions = constructCreateTemplateActions();
@@ -228,7 +245,7 @@ const QList<QAction *> DirectoryViewMenu::constructOpenOpActions()
             }
             if (info->isDir()) {
                 //add to bookmark option
-                if (!info->isVirtual() &&  !info->uri().startsWith("smb://") && !m_is_kydroid)
+                if (!info->isVirtual() &&  !info->uri().startsWith("smb://") && !m_is_kydroid && !m_is_filesafe)
                 {
                     l<<addAction(QIcon::fromTheme("bookmark-add-symbolic"), tr("Add to bookmark"));
                     connect(l.last(), &QAction::triggered, [=]() {
@@ -289,13 +306,28 @@ const QList<QAction *> DirectoryViewMenu::constructOpenOpActions()
                 l<<addAction(QIcon::fromTheme("document-open-symbolic"), tr("Open"));
                 connect(l.last(), &QAction::triggered, [=]() {
                     auto uri = m_selections.first();
-                    FileLaunchManager::openAsync(uri, false, false);
+                    if (m_is_network)
+                        m_top_window->goToUri(uri, true);
+                    else
+                        FileLaunchManager::openAsync(uri, false, false);
                 });
+
+                if (m_is_network)
+                    return l;
                 auto openWithAction = addAction(tr("Open with..."));
                 QMenu *openWithMenu = new QMenu(this);
                 // do not highlight application icons.
                 openWithMenu->setProperty("skipHighlightIconEffect", true);
-                auto recommendActions = FileLaunchManager::getRecommendActions(m_selections.first());
+                //auto recommendActions = FileLaunchManager::getRecommendActions(m_selections.first());
+                auto targetUri = FileUtils::getTargetUri(m_selections.first());
+                auto recommendActions = FileLaunchManager::getRecommendActions(targetUri);
+                //fix has default open app but no recommend actions issue, link to bug#61365
+                if (recommendActions.count() == 0)
+                {
+                    auto action = FileLaunchManager::getDefaultAction(m_selections.first());
+                    if (action != NULL && action->getAppInfoDisplayName().length() > 0)
+                        recommendActions.append(action);
+                }
                 for (auto action : recommendActions) {
                     action->setParent(openWithMenu);
                     openWithMenu->addAction(static_cast<QAction*>(action));
@@ -366,7 +398,7 @@ const QList<QAction *> DirectoryViewMenu::constructOpenOpActions()
 const QList<QAction *> DirectoryViewMenu::constructCreateTemplateActions()
 {
     QList<QAction *> l;
-    if (!m_is_favorite && m_selections.isEmpty()) {
+    if (!m_is_favorite && m_selections.isEmpty() && !m_is_filesafe) {
         auto createAction = new QAction(tr("New..."), this);
         if (m_is_cd) {
             createAction->setEnabled(false);
@@ -379,8 +411,7 @@ const QList<QAction *> DirectoryViewMenu::constructCreateTemplateActions()
             job.querySync();
         }
 
-        if (! info->canWrite())
-        {
+        if (!info->canWrite() && !m_is_ftp) {
             createAction->setEnabled(false);
         }
 
@@ -448,7 +479,11 @@ const QList<QAction *> DirectoryViewMenu::constructCreateTemplateActions()
                     g_object_unref(gtk_file);
                 }
                 subMenu->addSeparator();
+            } else {
+                qWarning()<<"the template dir is empty";
             }
+        } else {
+            qWarning()<<"the template dir is not exsit";
         }
 
         QList<QAction *> actions;
@@ -457,8 +492,8 @@ const QList<QAction *> DirectoryViewMenu::constructCreateTemplateActions()
         connect(actions.last(), &QAction::triggered, [=]() {
             //FileOperationUtils::create(m_directory);
             CreateTemplateOperation op(m_directory);
-            Peony::FileOperationErrorDialogConflict dlg;
-            connect(&op, &Peony::FileOperation::errored, &dlg, &Peony::FileOperationErrorDialogConflict::handle);
+            Peony::FileOperationErrorDialogWarning dlg;
+            connect(&op, &Peony::FileOperation::errored, &dlg, &Peony::FileOperationErrorDialogWarning::handle);
             op.run();
             auto targetUri = op.target();
             qDebug()<<"target:"<<targetUri;
@@ -469,8 +504,8 @@ const QList<QAction *> DirectoryViewMenu::constructCreateTemplateActions()
         connect(actions.last(), &QAction::triggered, [=]() {
             //FileOperationUtils::create(m_directory, nullptr, CreateTemplateOperation::EmptyFolder);
             CreateTemplateOperation op(m_directory, CreateTemplateOperation::EmptyFolder, tr("New Folder"));
-            Peony::FileOperationErrorDialogConflict dlg;
-            connect(&op, &Peony::FileOperation::errored, &dlg, &Peony::FileOperationErrorDialogConflict::handle);
+            Peony::FileOperationErrorDialogWarning dlg;
+            connect(&op, &Peony::FileOperation::errored, &dlg, &Peony::FileOperationErrorDialogWarning::handle);
             op.run();
             auto targetUri = op.target();
             qDebug()<<"target:"<<targetUri;
@@ -600,7 +635,7 @@ const QList<QAction *> DirectoryViewMenu::constructFileOpActions()
         QString homeUri = "file://" +  QStandardPaths::writableLocation(QStandardPaths::HomeLocation);
         bool hasStandardPath = FileUtils::containsStandardPath(m_selections);
         //qDebug() << "constructFileOpActions hasStandardPath:" <<hasStandardPath;
-        if (!m_selections.isEmpty() && !m_selections.contains(homeUri) && !m_is_recent) {
+        if (!m_selections.isEmpty() && !m_selections.contains(homeUri) && !m_is_recent && !m_is_filesafe) {
             if (!m_is_favorite) {
                 //code from lixiang for kydroid menu
                 if (m_is_kydroid) {
@@ -631,36 +666,60 @@ const QList<QAction *> DirectoryViewMenu::constructFileOpActions()
             if (m_is_kydroid)
                 return l;
 
-            if (!hasStandardPath && !m_is_recent && !m_is_favorite)
+            if (!hasStandardPath && !m_is_recent && !m_is_favorite && !m_is_filesafe)
             {
                 l<<addAction(QIcon::fromTheme("edit-cut-symbolic"), tr("Cut"));
                 connect(l.last(), &QAction::triggered, [=]() {
                     ClipboardUtils::setClipboardFiles(m_selections, true);
+                    m_view->repaintView();
                 });
             }
 
-            if (!m_is_recent && !m_is_favorite && !hasStandardPath) {
-                l<<addAction(QIcon::fromTheme("edit-delete-symbolic"), tr("Delete to trash"));
-                connect(l.last(), &QAction::triggered, [=]() {
-                    FileOperationUtils::trash(m_selections, true);
-                });
+            bool hasDeleteForever = false;
+            if (!m_is_recent && !m_is_favorite && !hasStandardPath && !m_is_filesafe) {
+                bool canTrash = true;
+                bool canDelete = true;
+                for (auto uri : m_selections) {
+                    auto info = FileInfo::fromUri(uri);
+                    if (! info->canTrash())
+                        canTrash = false;
+
+                    if (! info->canDelete())
+                        canDelete = false;
+                }
+                //fix bug#67932, file can not trash, use delete forever
+                if (canTrash && ! m_selections.first().startsWith("file:///data/usershare"))
+                {
+                    l<<addAction(QIcon::fromTheme("edit-delete-symbolic"), tr("Delete to trash"));
+                    connect(l.last(), &QAction::triggered, [=]() {
+                        FileOperationUtils::trash(m_selections, true);
+                    });
+                }
+                else if(m_can_delete && canDelete)
+                {
+                    hasDeleteForever = true;
+                    l<<addAction(QIcon::fromTheme("edit-clear-symbolic"), tr("Delete forever"));
+                    connect(l.last(), &QAction::triggered, [=]() {
+                        FileOperationUtils::executeRemoveActionWithDialog(m_selections);
+                    });
+                }
             }
 
-            if (m_is_favorite && m_can_delete) {
+            if (m_is_favorite && m_can_delete && !m_is_filesafe && !hasDeleteForever) {
                 l<<addAction(QIcon::fromTheme("edit-clear-symbolic"), tr("Delete forever"));
                 connect(l.last(), &QAction::triggered, [=]() {
                     FileOperationUtils::executeRemoveActionWithDialog(m_selections);
                 });
             }
 
-            if (m_selections.count() == 1 && ! hasStandardPath && !m_is_recent && !m_is_favorite) {
+            if (m_selections.count() == 1 && ! hasStandardPath && !m_is_recent && !m_is_favorite && !m_is_filesafe) {
                 l<<addAction(QIcon::fromTheme("document-edit-symbolic"), tr("Rename"));
                 connect(l.last(), &QAction::triggered, [=]() {
                     m_view->editUri(m_selections.first());
                 });
             }
         } else {
-            if (!m_is_recent && !m_is_favorite && !m_is_kydroid)
+            if (!m_is_recent && !m_is_favorite && !m_is_kydroid && !m_is_filesafe)
             {
                 auto pasteAction = addAction(QIcon::fromTheme("edit-paste-symbolic"), tr("Paste"));
                 l<<pasteAction;
@@ -678,12 +737,23 @@ const QList<QAction *> DirectoryViewMenu::constructFileOpActions()
                     }
                 });
             }
-            else if (m_is_recent)
+            else if (m_is_recent && m_selections.count() >0)
             {
                 //fix recent files can not be deleted issue
-                l<<addAction(QIcon::fromTheme("edit-clear-symbolic"), tr("Delete forever"));
+                l<<addAction(QIcon::fromTheme("edit-clear-symbolic"), tr("Delete"));
                 connect(l.last(), &QAction::triggered, [=]() {
-                    FileOperationUtils::executeRemoveActionWithDialog(m_selections);
+                    FileOperationUtils::remove(m_selections);
+                });
+
+                l<<addAction(QIcon::fromTheme("edit-copy-symbolic"), tr("Copy"));
+                connect(l.last(), &QAction::triggered, [=]() {
+                    QStringList selections;
+                    for(auto uri:m_selections)
+                    {
+                        uri = FileUtils::getTargetUri(uri);
+                        selections << uri;
+                    }
+                    ClipboardUtils::setClipboardFiles(selections, false);
                 });
             }
 
@@ -710,6 +780,14 @@ const QList<QAction *> DirectoryViewMenu::constructFileOpActions()
             //qDebug() << "Reverse select";
             m_view->invertSelections();
         });
+
+        if (m_is_search && m_selections.count() >0)
+        {
+            l<<addAction(QIcon::fromTheme("edit-copy-symbolic"), tr("Copy"));
+            connect(l.last(), &QAction::triggered, [=]() {
+                ClipboardUtils::setClipboardFiles(m_selections, false);
+            });
+        }
     }
 
     return l;
@@ -720,10 +798,21 @@ const QList<QAction *> DirectoryViewMenu::constructFilePropertiesActions()
     QList<QAction *> l;
 
     //fix select mutiple file in trash path show empty issue
-    if (m_selections.count() >1 && m_is_trash)
+    if (m_selections.count() > 1 && (m_is_trash || m_is_recent))
         return l;
 
     if (! m_is_search) {
+        //包含network的情况下，不显示属性选项
+        if (m_is_network) {
+            return l;
+        }
+
+        for (QString uri : m_selections) {
+            if (uri.startsWith("network://")) {
+                return l;
+            }
+        }
+
         l<<addAction(QIcon::fromTheme("preview-file"), tr("Properties"));
         connect(l.last(), &QAction::triggered, [=]() {
             //FIXME:
@@ -755,22 +844,32 @@ const QList<QAction *> DirectoryViewMenu::constructComputerActions()
 {
     QList<QAction *> l;
 
-    if (m_is_computer) {
-        /*
-        //FIXME: get the volume state and add action dynamicly.
-        if (m_selections.count() == 1) {
-            l<<addAction(tr("&Umount"));
-            connect(l.last(), &QAction::triggered, [=](){
-                VolumeManager::unmount(m_selections.first());
-            });
-            l<<addAction(tr("&Format"));
-            connect(l.last(), &QAction::triggered, [=](){
-                //FIXME:
-                //add format function?
-                //maybe put it in plugin...
-            });
+    if (m_is_computer && m_selections.count() == 1) {
+        QString uri = m_selections.first();
+
+        auto info = FileInfo::fromUri(uri);
+        if (info->displayName().isEmpty() || info->targetUri().isEmpty()) {
+            FileInfoJob j(info);
+            j.querySync();
+        }
+
+        auto mount = VolumeManager::getMountFromUri(info->targetUri());
+        //fix bug#52491, CDROM and DVD can format issue
+        if (nullptr != mount) {
+            QString unixDevice = FileUtils::getUnixDevice(info->uri());
+            if (! unixDevice.isNull() && ! unixDevice.contains("/dev/sr")
+                && info->isVolume() && info->canUnmount()) {
+                l<<addAction(QIcon::fromTheme("preview-file"), tr("format"), [=] () {
+                    // FIXME:// refactory Format_Dialog
+                    Format_Dialog* fd  = new Format_Dialog(info->uri(), nullptr, m_view);
+                    fd->show();
+                });
+
+                if (!mount) {
+                    l.last()->setEnabled(false);
+                }
+            }
         }
-        */
     }
 
     return l;
@@ -862,7 +961,7 @@ const QList<QAction *> DirectoryViewMenu::constructSearchActions()
                 {
                     QMessageBox::warning(nullptr,
                                          tr("Error"),
-                                         tr("File:\"%1\ is not exist, did you moved or deleted it?").arg(QUrl(uri).path()));
+                                         tr("File:\"%1\" is not exist, did you moved or deleted it?").arg(QUrl(uri).path()));
                 }
             }
         });
@@ -880,11 +979,25 @@ const QList<QAction *> DirectoryViewMenu::constructMenuPluginActions()
 
         for (auto id : pluginIds) {
             auto plugin = MenuPluginManager::getInstance()->getPlugin(id);
-            auto actions = plugin->menuActions(MenuPluginInterface::DirectoryView, m_directory, m_selections);
-            l<<actions;
-            for (auto action : actions) {
-                action->setParent(this);
-                addAction(action);
+
+            if(m_is_filesafe) {
+                if(plugin->name() == tr("Peony-Qt filesafe menu Extension")) {
+                    auto actions = plugin->menuActions(MenuPluginInterface::DirectoryView, m_directory, m_selections);
+                    l<<actions;
+                    for (auto action : actions) {
+                        action->setParent(this);
+                        addAction(action);
+                    }
+                }
+            } else {
+                if(plugin->name() != tr("Peony-Qt filesafe menu Extension")) {
+                    auto actions = plugin->menuActions(MenuPluginInterface::DirectoryView, m_directory, m_selections);
+                    l<<actions;
+                    for (auto action : actions) {
+                        action->setParent(this);
+                        addAction(action);
+                    }
+                }
             }
         }
     }
