@@ -53,6 +53,7 @@
 
 #include <QProxyStyle>
 #include <QStyleOptionToolButton>
+#include <syslog.h>
 
 using namespace Peony;
 
@@ -81,8 +82,6 @@ LocationBar::LocationBar(QWidget *parent) : QWidget(parent)
     setAttribute(Qt::WA_Hover);
     setMouseTracking(true);
 
-    setToolTip(tr("click the blank area for edit"));
-
     setStyleSheet("padding-right: 15;"
                   "margin-left: 2");
     m_styled_edit = new QLineEdit;
@@ -177,7 +176,7 @@ void LocationBar::setRootUri(const QString &uri)
                         // add buttons
                         clearButtons();
                         for (auto info : m_buttons_info) {
-                            addButton(info.get()->uri(), true, true);
+                            addButton(info.get()->uri().toLocal8Bit(), true, true);
                         }
                         doLayout();
                     }
@@ -272,6 +271,7 @@ void LocationBar::addButton(const QString &uri, bool setIcon, bool setMenu)
     button->setPopupMode(QToolButton::MenuButtonPopup);
 
     auto displayName = FileUtils::getFileDisplayName(uri);
+    button->setToolTip(displayName);
     m_buttons.insert(uri, button);
     if (m_current_uri.startsWith("search://")) {
         QString nameRegexp = SearchVFSUriParser::getSearchUriNameRegexp(m_current_uri);
@@ -291,6 +291,7 @@ void LocationBar::addButton(const QString &uri, bool setIcon, bool setMenu)
         button->setIcon(icon);
     }
 
+	button->setStyleSheet("QToolButton{padding-left: 13px; padding-right: 13px}");
     if (!url.fileName().isEmpty()) {
         if (FileUtils::getParentUri(uri).isNull()) {
             setMenu = false;
@@ -299,11 +300,14 @@ void LocationBar::addButton(const QString &uri, bool setIcon, bool setMenu)
         m_current_uri = uri.left(uri.lastIndexOf("/")+1) + displayName;
     } else {
         if (uri == "file:///") {
-            auto text = FileUtils::getFileDisplayName("computer:///root.link");
-            if (text.isNull()) {
-                text = tr("File System");
-            }
+//            auto text = FileUtils::getFileDisplayName("computer:///root.link");
+//            if (text.isNull()) {
+//                text = tr("File System");
+//            }
+            //fix bug#47597, show as root.link issue
+            QString text = tr("File System");
             button->setText(text);
+	        button->setStyleSheet("QToolButton{padding-left: 15px; padding-right: 15px}");
         } else {
             button->setText(displayName);
         }
@@ -372,7 +376,7 @@ void LocationBar::addButton(const QString &uri, bool setIcon, bool setMenu)
     connect(button, &QWidget::customContextMenuRequested, this, [=](){
         QMenu menu;
         FMWindowIface *windowIface = dynamic_cast<FMWindowIface *>(this->topLevelWidget());
-        auto copy = menu.addAction(QIcon::fromTheme("edit-copy-symbolic"), tr("&Copy Directory"));
+        auto copy = menu.addAction(QIcon::fromTheme("edit-copy-symbolic"), tr("Copy Directory"));
 
         menu.addAction(QIcon::fromTheme("tab-new-symbolic"), tr("Open In New Tab"), [=](){
             windowIface->addNewTabs(QStringList()<<uri);
@@ -416,7 +420,7 @@ void LocationBar::paintEvent(QPaintEvent *e)
     fopt.state |= QStyle::State_HasFocus;
     //fopt.state.setFlag(QStyle::State_HasFocus);
     fopt.rect.adjust(-2, 0, 0, 0);
-    fopt.palette.setColor(QPalette::Highlight, fopt.palette.base().color());
+    fopt.palette.setColor(QPalette::Highlight, fopt.palette.button().color());
     fopt.palette.setColor(QPalette::Base, fopt.palette.window().color());
 
     style()->drawPrimitive(QStyle::PE_PanelLineEdit, &fopt, &p, this);
